(* ::Package:: *)

(*:Mathematica Version: 8.01 *)

(*:Package Version: 1.09 *)

(*:Version History: 1.01 (2007.07.30 - 2007.09.30)
                    1.02 (2007.10.01 - 2007.10.13)
                    1.03 (2007.10.14 - 2009.07.18)
                    1.04 (2009.07.19 - 2012.12.16)
                    1.05 (2012.12.17 - 2013.01.06)
                    1.06 (2013.01.07 - 2013.10.30)
                    1.07 (2013.10.31 - 2014.01.18)MeijerHypergeometricG
                    1.08 (2014.01.19 - 2015.06.15)
                    1.09 (2015.06.16 -     .  .  )
 *)

(*:Copyright: Copyright 2007.07-2015.06, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: SpecialFunctions`GeneralizedHypergeometric` *)

(*:Title: GeneralizedHypergeometric *)

(*:Keywords: Hypergeometric2F2, Hypergeometric3F2, 
             Hypergeometric3F3, Hypergeometric4F3,
             MittagLefflerE, MittagLefflerS, MittagLefflerC, WrightW,
             MeijerHypergeometricG *)

(*:Summary:
    This package implements the functions of relation 
    Pochhammer's generalized hypergeometric functions, Meijer's 
    G-functions and Mittag-Leffler's functions. *)

(******************************************************************************)

BeginPackage["SpecialFunctions`GeneralizedHypergeometric`"]



Hypergeometric2F2::usage =
"Hypergeometric2F2[a1,a2,b1,b2,z] gives the Pochhammer's
 generalized hypergeometric functions 2F2."

Hypergeometric3F2::usage =
"Hypergeometric3F2[a1,a2,a3,b1,b2,z] gives the Pochhammer's
 generalized hypergeometric functions 3F2."

Hypergeometric3F3::usage =
"Hypergeometric3F3[a1,a2,a3,b1,b2,b3,z] gives the Pochhammer's
 generalized hypergeometric functions 3F3."

Hypergeometric4F3::usage =
"Hypergeometric4F3[a1,a2,a3,a4,b1,b2,b3,z] gives the Pochhammer's
 generalized hypergeometric functions 4F3."

MittagLefflerE::usage =
"MittagLefflerE[a,b,z] gives the Mittag-Leffler's functions."

MittagLefflerS::usage =
"MittagLefflerS[a,b,z] gives the sine type of the Mittag-Leffler's
 functions."

MittagLefflerC::usage =
"MittagLefflerC[a,b,z] gives the cosine type of the Mittag-Leffler's
 functions."

WrightW::usage =
"WrightW[a,b,z] gives the cosine type of the Wright's functions."

MeijerHypergeometricG::usage =
"MeijerHypergeometricG[{{a1},{ap}},{{b1},{bq}},z] gives the Meijer's
 G-functions.
 Here {a1},{ap},{b1},and {bq} is lists of number."



Begin["SpecialFunctions`GeneralizedHypergeometric`Private`"]



Attributes[Hypergeometric2F2]={Listable}

Attributes[Hypergeometric3F2]={Listable}

Attributes[Hypergeometric3F3]={Listable}

Attributes[Hypergeometric4F3]={Listable}

Attributes[MittagLefflerE]={Listable}

Attributes[MittagLefflerS]={Listable}

Attributes[MittagLefflerC]={Listable}

Attributes[WrightW]={Listable}




(******************************************************************************)
(*                      Hypergeometric2F2[a1,a2,b1,b2,z]                      *)
(******************************************************************************)

Region2F2[z_]:=If[Abs[z]<=10 || Abs[Arg[z]]<=N[Pi/5],True,False];

setPrec2F2[z_]:=
  If[Precision[z]<Infinity,
  SetPrecision[z*10^18,Floor[Sqrt[400+Abs[z]^2]]]/(10^18),z];



Hypergeometric2F2[a1_,a2_,b1_,b2_,z_]:=
  HypergeometricPFQ[{a1,a2},{b1,b2},z]/;
  (Precision[z]<Infinity && Region2F2[z]===True)

Hypergeometric2F2[a1_,a2_,b1_,b2_,z_]:=
  Module[{s=0,so=-1,k1=0,a1f,a2f,b1f,b2f,nf,zf,spA1,spA2,spB1,spB2,spZ},
      a1f=1;a2f=1;b1f=1;b2f=1;nf=1;zf=1;spA1=setPrec2F2[a1];
      spA2=setPrec2F2[a2];spB1=setPrec2F2[b1];spB2=setPrec2F2[b2];
      spZ=setPrec2F2[z];
    While[so!=s,so=s; s+=(a1f*a2f/(b1f*b2f*nf)*zf);
      a1f*=(spA1+k1);a2f*=(spA2+k1);b1f*=(spB1+k1);b2f*=(spB2+k1);
      nf*=(k1+1);zf*=spZ;k1++];s]/;
      (Precision[z]<Infinity && Region2F2[z]===False && Abs[z]<=70)

Hypergeometric2F2[a1_,a2_,b1_,b2_,z_]:=
  Gamma[b1]*Gamma[b2]*Gamma[a2-a1]/(Gamma[a2]*Gamma[b1-a1]*Gamma[b2-a1])*
  finite3F1First[a1,a2,b1,b2,z]*(-z)^(-a1)+
  Gamma[b1]*Gamma[b2]*Gamma[a1-a2]/(Gamma[a1]*Gamma[b1-a2]*Gamma[b2-a2])*
  finite3F1Second[a1,a2,b1,b2,z]*(-z)^(-a2)+
  Gamma[b1]*Gamma[b2]/(Gamma[a1]*Gamma[a2])*z^(a1+a2-b1-b2)*Exp[z]*
  hyp2F2Series[a1,a2,b1,b2,z]/;
  (Precision[z]<Infinity && Region2F2[z]===False && Abs[z]>70)



finite3F1First[a1_,a2_,b1_,b2_,z_]:=
  Module[{x,ax1,ax2,bx1,bx2},
  (finite3F1First1[ax1,ax2,bx1,bx2,x]+finite3F1First2[ax1,ax2,bx1,bx2,x])/2/.
  {x->-1/z,ax1->a1,ax2->a2,bx1->b1,bx2->b2}];

finite3F1First1[ax1_Symbol,ax2_Symbol,bx1_Symbol,bx2_Symbol,x_Symbol]:=
  finite3F1First1[ax1,ax2,bx1,bx2,x]=
  Normal[Series[HypergeometricPFQ[{ax1,ax1-bx1+1,ax1-bx2+1},{ax1-ax2+1},x],{x,0,12}]];

finite3F1First2[ax1_Symbol,ax2_Symbol,bx1_Symbol,bx2_Symbol,x_Symbol]:=
  finite3F1First2[ax1,ax2,bx1,bx2,x]=
  Normal[Series[HypergeometricPFQ[{ax1,ax1-bx1+1,ax1-bx2+1},{ax1-ax2+1},x],{x,0,13}]];


finite3F1Second[a1_,a2_,b1_,b2_,z_]:=
  Module[{x,ax1,ax2,bx1,bx2},
  (finite3F1Second1[ax1,ax2,bx1,bx2,x]+finite3F1Second2[ax1,ax2,bx1,bx2,x])/2/.
  {x->-1/z,ax1->a1,ax2->a2,bx1->b1,bx2->b2}];

finite3F1Second1[ax1_Symbol,ax2_Symbol,bx1_Symbol,bx2_Symbol,x_Symbol]:=
  finite3F1Second1[ax1,ax2,bx1,bx2,x]=
  Normal[Series[HypergeometricPFQ[{ax2,ax2-bx1+1,ax2-bx2+1},{ax2-ax1+1},x],{x,0,12}]];

finite3F1Second2[ax1_Symbol,ax2_Symbol,bx1_Symbol,bx2_Symbol,x_Symbol]:=
  finite3F1Second2[ax1,ax2,bx1,bx2,x]=
  Normal[Series[HypergeometricPFQ[{ax2,ax2-bx1+1,ax2-bx2+1},{ax2-ax1+1},x],{x,0,13}]];



hyp2F2Series[a1_,a2_,b1_,b2_,z_]:=
  Module[{s=0,so=-1,z1=(1/z),k1=0,zf},zf=1;
    While[(Chop[so-s,10^-8]=!=0 && k1<25),so=s;
    s+=N[zf*hyp2F2SerCoeff[a1,a2,b1,b2,k1]];
    zf*=z1;k1++];s]/;(Precision[z]<Infinity)

 	
hyp2F2SerCoeff[a1_,a2_,b1_,b2_,0]:=1;

hyp2F2SerCoeff[a1_,a2_,b1_,b2_,1]:=(a1+a2-1)*(a1+a2-b1-b2)-a1*a2+b1*b2;

hyp2F2SerCoeff[a1_,a2_,b1_,b2_,k_Integer/;(k>=2)]:=
  hyp2F2SerCoeff[a1,a2,b1,b2,k]=
  ((1-b1-b2+2*a1+a1^2+2*a2+a2^2-(a1+a2)*(b1+b2)+a1*a2+b1*b2)/k+
  (2*(b1+b2)-3*(a1+a2)-3)+2*k)*hyp2F2SerCoeff[a1,a2,b1,b2,k-1]-
  (k-a1-a2+b1-1)*(k-a1-a2+b2-1)*(k-a1-a2+b1+b2-2)*
  hyp2F2SerCoeff[a1,a2,b1,b2,k-2];


(******************************************************************************)
(*                    Hypergeometric3F2[a1,a2,a3,b1,b2,z]                     *)
(******************************************************************************)

Hypergeometric3F2[a1_,a2_,a3_,b1_,b2_,z_]:=
  HypergeometricPFQ[{a1,a2,a3},{b1,b2},z]/;
  ((Abs[z+0.05]>=1.05 || Abs[z-0.2]<=0.8 || Re[z]>=0.5) && 
  Abs[Im[z]]>0 && Precision[z]<Infinity)


Hypergeometric3F2[a1_,a2_,a3_,b1_,b2_,z_]:=
  1+a1*a2*a3*z/(b1*b2*(1+hypGeo3F2ContFrac[a1,a2,a3,b1,b2,z]))/;
  (Abs[z-0.2]<=0.8 && N[Im[z]]===0. && Precision[z]<Infinity)

hypGeo3F2ContFrac[a1_,a2_,a3_,b1_,b2_,z_]:=
  Fold[((-(a1+#2) (a2+#2) (a3+#2) z/((1+#2) (b1+#2) (b2+#2)))/
  (1+(a1+#2) (a2+#2) (a3+#2) z/((1+#2) (b1+#2) (b2+#2))+#1))&,0,Range[100,1,-1]];


Hypergeometric3F2[a1_,a2_,a3_,b1_,b2_,z_]:=
  Module[{s=0,so=-1,z1=(z/(z-1)),k1=0,gf,pof,zf},
    gf=1;pof=1;zf=1;
    While[(Chop[so-s,10^-8]=!=0 && k1<100),so=s;
    s+=(zf/gf*pof*HypergeometricPFQ[{-k1,a2,a3},{b1,b2},1]);
    gf*=(k1+1);pof*=(a1+k1);zf*=z1;k1++];s]/((1-z)^a1)/;
   (((Abs[z+0.05]<1.05 && Abs[z-0.2]>0.8 && Re[z]<0.5 && Abs[Im[z]]>0) || 
   (-1<=Re[z]<-0.6 && N[Im[z]]===0.)) && Precision[z]<Infinity)


Hypergeometric3F2[a1_,a2_,a3_,b1_,b2_,z_]:=
  (Hypergeometric3F2[a1,a1-b1+1,a1-b2+1,a1-a2+1,a1-a3+1,1/z]*
  (-z)^(-a1)*Gamma[a2-a1]*Gamma[a3-a1]/
  (Gamma[b1-a1]*Gamma[b2-a1]*Gamma[a2]*Gamma[a3])+
  Hypergeometric3F2[a2,a2-b1+1,a2-b2+1,a2-a1+1,a2-a3+1,1/z]*
  (-z)^(-a2)*Gamma[a1-a2]*Gamma[a3-a2]/
  (Gamma[b1-a2]*Gamma[b2-a2]*Gamma[a1]*Gamma[a3])+
  Hypergeometric3F2[a3,a3-b1+1,a3-b2+1,a3-a1+1,a3-a2+1,1/z]*
  (-z)^(-a3)*Gamma[a1-a3]*Gamma[a2-a3]/
  (Gamma[b1-a3]*Gamma[b2-a3]*Gamma[a1]*Gamma[a2]))*Gamma[b1]*Gamma[b2]/;
  ((Re[z]<-1 || Re[z]>1) && N[Im[z]]===0. && Precision[z]<Infinity)


(******************************************************************************)
(*                   Hypergeometric3F3[a1,a2,a3,b1,b2,b3,z]                   *)
(******************************************************************************)

Region3F3[z_]:=If[Abs[z]<=5 || Abs[Arg[z]]<=N[Pi/5],True,False];

setPrec3F3[z_]:=
  If[Precision[z]<Infinity,
  SetPrecision[z*10^32,Floor[Sqrt[400+Abs[z]^2]]]/(10^32),z];



Hypergeometric3F3[a1_,a2_,a3_,b1_,b2_,b3_,z_]:=
  HypergeometricPFQ[{a1,a2,a3},{b1,b2,b3},z]/;
  (Precision[z]<Infinity && Region3F3[z]===True)

Hypergeometric3F3[a1_,a2_,a3_,b1_,b2_,b3_,z_]:=
  Module[{s=0,so=-1,k1=0,a1f,a2f,a3f,b1f,b2f,b3f,nf,zf,
          spA1,spA2,spA3,spB1,spB2,spB3,spZ},
      a1f=1;a2f=1;a3f=1;b1f=1;b2f=1;b3f=1;nf=1;zf=1;spA1=setPrec3F3[a1];
      spA2=setPrec3F3[a2];spA3=setPrec3F3[a3];spB1=setPrec3F3[b1];
      spB2=setPrec3F3[b2];spB3=setPrec3F3[b3];spZ=setPrec3F3[z];
    While[so!=s,so=s; s+=(a1f*a2f*a3f/(b1f*b2f*b3f*nf)*zf);
      a1f*=(spA1+k1);a2f*=(spA2+k1);a3f*=(spA3+k1);b1f*=(spB1+k1);
      b2f*=(spB2+k1);b3f*=(spB3+k1);nf*=(k1+1);zf*=spZ;k1++];s]/;
      (Precision[z]<Infinity && Region3F3[z]===False)


(******************************************************************************)
(*                 Hypergeometric4F3[a1,a2,a3,a4,b1,b2,b3,z]                  *)
(******************************************************************************)

Hypergeometric4F3[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  HypergeometricPFQ[{a1,a2,a3,a4},{b1,b2,b3},z]/;
  (Precision[z]<Infinity && Abs[z-0.01]<1 && Abs[z]<=1 && 
  h4F3PoleQ[a1,a2,a3,a4,b1,b2,b3,z]===False)


Hypergeometric4F3[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  Module[{s=0,so=-1,z1=(z/(z-1)),k1=0,gf,pof,zf},gf=1;pof=1;zf=1;
    While[k1<25,so=s;s+=(zf/gf*pof*HypergeometricPFQ[{-k1,a2,a3,a4},{b1,b2,b3},1]);
    gf*=(k1+1);pof*=(a1+k1);zf*=z1;k1++];s]/((1-z)^a1)/;
   (Precision[z]<Infinity && Abs[z-0.01]>=1 && Abs[z]<=1 && 
   h4F3PoleQ[a1,a2,a3,a4,b1,b2,b3,z]===False)


Hypergeometric4F3[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  (Hypergeometric4F3[a1,a1-b1+1,a1-b2+1,a1-b3+1,a1-a2+1,a1-a3+1,a1-a4+1,1/z]*(-z)^(-a1)*
  Gamma[a2-a1]*Gamma[a3-a1]*Gamma[a4-a1]/
  (Gamma[b1-a1]*Gamma[b2-a1]*Gamma[b3-a1]*Gamma[a2]*Gamma[a3]*Gamma[a4])+
  Hypergeometric4F3[a2,a2-b1+1,a2-b2+1,a2-b3+1,a2-a1+1,a2-a3+1,a2-a4+1,1/z]*(-z)^(-a2)*
  Gamma[a1-a2]*Gamma[a3-a2]*Gamma[a4-a2]/
  (Gamma[b1-a2]*Gamma[b2-a2]*Gamma[b3-a2]*Gamma[a1]*Gamma[a3]*Gamma[a4])+
  Hypergeometric4F3[a3,a3-b1+1,a3-b2+1,a3-b3+1,a3-a1+1,a3-a2+1,a3-a4+1,1/z]*(-z)^(-a3)*
  Gamma[a1-a3]*Gamma[a2-a3]*Gamma[a4-a3]/
  (Gamma[b1-a3]*Gamma[b2-a3]*Gamma[b3-a3]*Gamma[a1]*Gamma[a2]*Gamma[a4])+
  Hypergeometric4F3[a4,a4-b1+1,a4-b2+1,a4-b3+1,a4-a1+1,a4-a2+1,a4-a3+1,1/z]*(-z)^(-a4)*
  Gamma[a1-a4]*Gamma[a2-a4]*Gamma[a3-a4]/
  (Gamma[b1-a4]*Gamma[b2-a4]*Gamma[b3-a4]*Gamma[a1]*Gamma[a2]*Gamma[a3]))*
  Gamma[b1]*Gamma[b2]*Gamma[b3]/;
  (Precision[z]<Infinity && Abs[z]>1 && h4F3PoleQ[a1,a2,a3,a4,b1,b2,b3,z]===False)


Hypergeometric4F3[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  1+a1*a2*a3*a4*z/(b1*b2*b3*(1+hypGeo4F3ContFrac[a1,a2,a3,a4,b1,b2,b3,z]))/;
  (Precision[z]<Infinity && h4F3PoleQ[a1,a2,a3,a4,b1,b2,b3,z]===True)

hypGeo4F3ContFrac[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  Fold[((-(a1+#2) (a2+#2) (a3+#2) (a4+#2) z/((1+#2) (b1+#2) (b2+#2) (b3+#2)))/
  (1+(a1+#2) (a2+#2) (a3+#2) (a4+#2) z/((1+#2) (b1+#2) (b2+#2) (b3+#2))+#1))&,
  0,Range[100,1,-1]];


h4F3PoleQ[a1_,a2_,a3_,a4_,b1_,b2_,b3_,z_]:=
  If[(Abs[Im[z]]===0 && Abs[1-100^(1/(b1+b2+b3-a1-a2-a3-a4))]<Re[z]<=1 && 
  Re[b1+b2+b3-a1-a2-a3-a4]<0),True,False];


(******************************************************************************)
(*                              mlNonPosiIntQ[n]                              *)
(******************************************************************************)

mlNonPosiIntQ[n_]:=
  If[(N[Im[n]]===0. && MemberQ[{0.,0.+0. I},N[n+Abs[Floor[Re[n]]]]]),
  True,False];



(******************************************************************************)
(*                           MittagLefflerE[a,b,z]                            *)
(******************************************************************************)

MittagLefflerE[a_,z_]:=MittagLefflerE[a,1,z];



MittagLefflerE[a_,b_,z_]:=
  Module[{max=200,prec=50,ap,bp,zp,powerL},
   Block[{$MinPrecision=prec},
    ap=SetPrecision[a,prec];
    bp=SetPrecision[b,prec];
    zp=SetPrecision[z,prec];
    powerL=Table[zp^j,{j,1,max,1}];
   (emlTaylorCoef[ap,bp,0]+emlPadeCoefA[ap,bp,max].powerL)/
   (1+emlPadeCoefB[ap,bp,max].powerL)]]/;
  (Precision[z]<Infinity && Abs[Arg[a]]<N[Pi/2])



emlTaylorCoef[a_,b_,n_]:=
  If[mlNonPosiIntQ[a*n+b],0,1/Gamma[a*n+b]];



emlPadeMatrixA[a_,b_,v_Integer?Positive]:=
  Table[If[j-k<0,0,emlTaylorCoef[a,b,j-k]],{j,0,v,1},{k,0,v,1}];

emlPadeMatrixB[a_,b_,v_Integer?Positive]:=
  Table[emlTaylorCoef[a,b,v+j-k],{j,0,v-1,1},{k,0,v-1,1}];


emlPadeCoefA[a_,b_,v_Integer?Positive]:=
  emlPadeCoefA[a,b,v]=
   Rest[Flatten[emlPadeMatrixA[a,b,v].Join[{1},emlPadeCoefB[a,b,v]]]];

emlPadeCoefB[a_,b_,v_Integer?Positive]:=
  emlPadeCoefB[a,b,v]=
   LinearSolve[emlPadeMatrixB[a,b,v],
   Table[-emlTaylorCoef[a,b,v+j],{j,1,v,1}]];



(******************************************************************************)
(*                           MittagLefflerS[a,b,z]                            *)
(******************************************************************************)

MittagLefflerS[a_,z_]:=MittagLefflerS[a,1,z];



MittagLefflerS[a_,b_,z_]:=
  Module[{max=200,prec=50,ap,bp,zp,powerL},
   Block[{$MinPrecision=prec},
    ap=SetPrecision[a,prec];
    bp=SetPrecision[b,prec];
    zp=SetPrecision[z,prec];
    powerL=Table[zp^j,{j,1,max,1}];
   (smlTaylorCoef[ap,bp,0]+smlPadeCoefA[ap,bp,max].powerL)/
   (1+smlPadeCoefB[ap,bp,max].powerL)]]/;
  (Precision[z]<Infinity && Abs[Arg[a]]<N[Pi/2])


smlTaylorCoef[a_,b_,n_]:=
  If[EvenQ[n],0,
   If[mlNonPosiIntQ[a*n+b],0,(-1)^((n-1)/2)/Gamma[a*n+b]]];


smlPadeMatrixA[a_,b_,v_Integer?Positive]:=
  Table[If[j-k<0,0,smlTaylorCoef[a,b,j-k]],{j,0,v,1},{k,0,v,1}];

smlPadeMatrixB[a_,b_,v_Integer?Positive]:=
  Table[smlTaylorCoef[a,b,v+j-k],{j,0,v-1,1},{k,0,v-1,1}];


smlPadeCoefA[a_,b_,v_Integer?Positive]:=
  smlPadeCoefA[a,b,v]=
  Rest[Flatten[smlPadeMatrixA[a,b,v].Join[{1},smlPadeCoefB[a,b,v]]]];

smlPadeCoefB[a_,b_,v_Integer?Positive]:=
  smlPadeCoefB[a,b,v]=
  LinearSolve[smlPadeMatrixB[a,b,v],
  Table[-smlTaylorCoef[a,b,v+j],{j,1,v,1}]];



(******************************************************************************)
(*                           MittagLefflerC[a,b,z]                            *)
(******************************************************************************)

MittagLefflerC[a_,z_]:=MittagLefflerC[a,1,z];



MittagLefflerC[a_,b_,z_]:=
  Module[{max=200,prec=50,ap,bp,zp,powerL},
   Block[{$MinPrecision=prec},
    ap=SetPrecision[a,prec];
    bp=SetPrecision[b,prec];
    zp=SetPrecision[z,prec];
    powerL=Table[zp^j,{j,1,max,1}];
   (cmlTaylorCoef[ap,bp,0]+cmlPadeCoefA[ap,bp,max].powerL)/
   (1+cmlPadeCoefB[ap,bp,max].powerL)]]/;
  (Precision[z]<Infinity && Abs[Arg[a]]<N[Pi/2])


cmlTaylorCoef[a_,b_,n_]:=
  If[OddQ[n],0,
   If[mlNonPosiIntQ[a*n+b],0,(-1)^(n/2)/Gamma[a*n+b]]];


cmlPadeMatrixA[a_,b_,v_Integer?Positive]:=
  Table[If[j-k<0,0,cmlTaylorCoef[a,b,j-k]],{j,0,v,1},{k,0,v,1}];

cmlPadeMatrixB[a_,b_,v_Integer?Positive]:=
  Table[cmlTaylorCoef[a,b,v+j-k],{j,0,v-1,1},{k,0,v-1,1}];


cmlPadeCoefA[a_,b_,v_Integer?Positive]:=
  cmlPadeCoefA[a,b,v]=
  Rest[Flatten[cmlPadeMatrixA[a,b,v].Join[{1},cmlPadeCoefB[a,b,v]]]];

cmlPadeCoefB[a_,b_,v_Integer?Positive]:=
  cmlPadeCoefB[a,b,v]=
  LinearSolve[cmlPadeMatrixB[a,b,v],
  Table[-cmlTaylorCoef[a,b,v+j],{j,1,v,1}]];



(******************************************************************************)
(*                               WrightW[a,b,z]                               *)
(******************************************************************************)

WrightW[a_,b_,z_]:=
  Module[{max=200,prec=50,ap,bp,zp,powerL},
   Block[{$MinPrecision=prec},
    ap=SetPrecision[a,prec];
    bp=SetPrecision[b,prec];
    zp=SetPrecision[z,prec];
    powerL=Table[zp^j,{j,1,max,1}];
   (wrightTaylorCoef[ap,bp,0]+wrightPadeCoefA[ap,bp,max].powerL)/
   (1+wrightPadeCoefB[ap,bp,max].powerL)]]/;
  (Precision[z]<Infinity)


wrightTaylorCoef[a_,b_,n_]:=
  If[mlNonPosiIntQ[a*n+b],0,1/(n!*Gamma[a*n+b])];


wrightPadeMatrixA[a_,b_,v_Integer?Positive]:=
  Table[If[j-k<0,0,
  wrightTaylorCoef[a,b,j-k]],{j,0,v,1},{k,0,v,1}];

wrightPadeMatrixB[a_,b_,v_Integer?Positive]:=
  Table[wrightTaylorCoef[a,b,v+j-k],{j,0,v-1,1},{k,0,v-1,1}];


wrightPadeCoefA[a_,b_,v_Integer?Positive]:=
  wrightPadeCoefA[a,b,v]=
  Rest[Flatten[wrightPadeMatrixA[a,b,v].Join[{1},wrightPadeCoefB[a,b,v]]]];

wrightPadeCoefB[a_,b_,v_Integer?Positive]:=
  wrightPadeCoefB[a,b,v]=
  LinearSolve[wrightPadeMatrixB[a,b,v],
  Table[-wrightTaylorCoef[a,b,v+j],{j,1,v,1}]];



(******************************************************************************)
(*              MeijerHypergeometricG[{{a1},{ap}},{{b1},{bq}},z]              *)
(******************************************************************************)

(* The Meijer-G function *)
MeijerHypergeometricG[{an_List,ap_List},{bm_List,bq_List},z_]:=
  Module[{lenP,lenQ},
    lenP=Length[Join[an,ap]];
    lenQ=Length[Join[bm,bq]];
     If[lenP===lenQ,
       If[Chop[Im[z]]===0 && 0<Re[z]<1 && realIndexMeijerGQ[{an,ap},{bm,bq}],
         Re[meijerHypG[{an,ap},{bm,bq},z]],
         meijerHypG[{an,ap},{bm,bq},z]],
       If[Chop[Im[z]]===0 && Re[z]>0 && realIndexMeijerGQ[{an,ap},{bm,bq}],
         Re[MeijerG[SetPrecision[{an,ap},50],SetPrecision[{bm,bq},50],SetPrecision[z,50]]],
         MeijerG[SetPrecision[{an,ap},50],SetPrecision[{bm,bq},50],SetPrecision[z,50]]]]]/;
   (Precision[z]<Infinity)

realIndexMeijerGQ[{an_List,ap_List},{bm_List,bq_List}]:=
  Cases[Im[Flatten[{{an,ap},{bm,bq}}]],Except[0]]==={};

meijerHypG[{an_List,ap_List},{bm_List,bq_List},z_]:=
  Sum[powerFactorMG[k,{an,ap},{bm,bq},z]*
      seriesTermMG[k,{an,ap},{bm,bq},z],{k,1,Length[bm]}]/;
  (FreeQ[Map[MemberQ[{0.,0.+0. I},FractionalPart[N[#]]]&,
   Apply[Subtract,Permutations[bm,{2}],{1}]],True])

(* Power factor of the Meijer-G function *)
powerFactorMG[k_Integer?Positive,{an_List,ap_List},{bm_List,bq_List},z_]:=
  Apply[Times,Gamma[Delete[bm,k]-bm[[k]]]]*
  Apply[Times,Gamma[1-an+bm[[k]]]]/Apply[Times,Gamma[ap-bm[[k]]]]/
  Apply[Times,Gamma[1-bq+bm[[k]]]]*z^bm[[k]];

(* Series term of the Meijer-G function *)
convergeAt1[{an_List,ap_List},{bm_List,bq_List}]:=
  convergeAt1[{an,ap},{bm,bq}]=
  If[MeijerG[{an,ap},{bm,bq},1]===Indeterminate,False,True];

seriesTermMG[k_Integer?Positive,{an_List,ap_List},{bm_List,bq_List},z_]:=
  If[convergeAt1[{an,ap},{bm,bq}],
    normalSeriesTermMG[k,{an,ap},{bm,bq},z],
    specialSeriesTermMG[k,{an,ap},{bm,bq},z]];

normalSeriesTermMG[k_Integer?Positive,{an_List,ap_List},{bm_List,bq_List},z_]:=
  Module[{iMax=600,prec=500,spAN,spAP,spBM,spBQ,powerL},
    {spAN,spAP,spBM,spBQ}=SetPrecision[{an,ap,bm,bq},prec];
    powerL=Table[SetPrecision[z,prec]^i,{i,1,iMax,1}];
  (coefSerTermMG[0,k,{spAN,spAP},{spBM,spBQ}]+
   stmgPadeCoeA[iMax,k,{spAN,spAP},{spBM,spBQ}].powerL)/
  (1+stmgPadeCoeB[iMax,k,{spAN,spAP},{spBM,spBQ}].powerL)];

  (*** caution : theta < argm ***)
specialSeriesTermMG[k_Integer?Positive,{an_List,ap_List},{bm_List,bq_List},z_]:=
  Module[{theta=ArcTan[0.0125]},
    If[0<=Arg[z-1]<theta,
      specialSeriesTermMG[k,{an,ap},{bm,bq},1+Abs[z-1] Exp[theta*I]],
    If[-theta<Arg[z-1]<0,
      specialSeriesTermMG[k,{an,ap},{bm,bq},1+Abs[z-1] Exp[-theta*I]],
      primarySeriesTermMG[k,{an,ap},{bm,bq},z]]]];

primarySeriesTermMG[k_Integer?Positive,{an_List,ap_List},{bm_List,bq_List},z_]:=
  Module[{argm=ArcTan[0.02],iMax,prec,spAN,spAP,spBM,spBQ,powerL},
    iMax=If[Abs[Arg[z]]<argm && Abs[z]>1,1200,600];
    prec=If[Abs[Arg[z]]<argm && Abs[z]>1,1000,500];
    {spAN,spAP,spBM,spBQ}=SetPrecision[{an,ap,bm,bq},prec];
    powerL=Table[SetPrecision[z,prec]^i,{i,1,iMax,1}];
  (coefSerTermMG[0,k,{spAN,spAP},{spBM,spBQ}]+
   stmgPadeCoeA[iMax,k,{spAN,spAP},{spBM,spBQ}].powerL)/
  (1+stmgPadeCoeB[iMax,k,{spAN,spAP},{spBM,spBQ}].powerL)];

(* Coefficient of power series *)
coefSerTermMG[0,k_,{an_,ap_},{bm_,bq_}]:=1;

coefSerTermMG[i_/;(IntegerQ[i] && i>0),k_,{an_,ap_},{bm_,bq_}]:=
  coefSerTermMG[i,k,{an,ap},{bm,bq}]=
  Module[{listA,listB,lenN,lenM,lenP},
    listA=Join[an,ap];
    listB=Join[bm,bq];
    lenN=Length[an];
    lenM=Length[bm];
    lenP=Length[Join[an,ap]];
   Apply[Times,Pochhammer[1-listA+bm[[k]],i]]/
   Apply[Times,Pochhammer[1-listB+bm[[k]],i]]*
   (-1)^(i*(lenP-lenM-lenN))];

(* Matrix of Pade approximation *)
stmgPadeMatA[i_Integer?Positive,k_,{an_,ap_},{bm_,bq_}]:=
  Table[If[j-h<0,0,coefSerTermMG[j-h,k,{an,ap},{bm,bq}]],
  {j,0,i,1},{h,0,i,1}];

stmgPadeMatB[i_Integer?Positive,k_,{an_,ap_},{bm_,bq_}]:=
  Table[coefSerTermMG[i+j-h,k,{an,ap},{bm,bq}],
  {j,0,i-1,1},{h,0,i-1,1}];

stmgPadeCoeA[i_Integer?Positive,k_,{an_,ap_},{bm_,bq_}]:=
  stmgPadeCoeA[i,k,{an,ap},{bm,bq}]=
  Rest[Flatten[Dot[stmgPadeMatA[i,k,{an,ap},{bm,bq}],
  Join[{1},stmgPadeCoeB[i,k,{an,ap},{bm,bq}]]]]];

stmgPadeCoeB[i_Integer?Positive,k_,{an_,ap_},{bm_,bq_}]:=
  stmgPadeCoeB[i,k,{an,ap},{bm,bq}]=
  LinearSolve[stmgPadeMatB[i,k,{an,ap},{bm,bq}],
  Table[-coefSerTermMG[i+j,k,{an,ap},{bm,bq}],{j,1,i,1}]];



(***********************************************************)

End[]

Protect[Hypergeometric2F2, Hypergeometric3F2, 
        Hypergeometric3F3, Hypergeometric4F3, 
        MittagLefflerE, MittagLefflerS, MittagLefflerC, WrightW, 
        MeijerHypergeometricG]

EndPackage[]
